@@ -63,7 +63,7 @@ import numpy as np
 import numpy.ma as ma
 
 
-__version__ = '0.7.0'
+__version__ = '0.8.0'
 
 
 _SCALAR_KEY_TYPES = (int, np.integer)
@@ -288,6 +288,8 @@ class ConsumerNode(Node):
                 if any(chunk is QUEUE_FINISHED for chunk in input_chunks):
                     break
                 self.output(self.process_chunks(input_chunks))
+            # Finalise the final chunk (process_chunks does this for all
+            # but the last chunk).
             self.output(self.finalise())
         except:
             self.abort()
@@ -557,7 +559,7 @@ class ConstantArray(Array):
     Returns
     -------
     Array
-        An Array entirely filled with ones.
+        An Array entirely filled with 'value'.
 
     """
     def __init__(self, shape, value=0.0, dtype=None):
@@ -938,12 +940,20 @@ class ArrayStack(Array):
     """
     An Array made from a homogeneous array of other Arrays.
 
+    Parameters
+    ----------
+    stack : array-like
+        The array of Arrays to be stacked, where each Array must be of
+        the same shape.
+
     """
     def __init__(self, stack):
+        stack = np.require(stack, dtype='O')
         first_array = stack.flat[0]
         item_shape = first_array.shape
         dtype = first_array.dtype
         fill_value = first_array.fill_value
+
         if np.issubdtype(dtype, np.floating):
             def fill_value_ok(array):
                 return array.fill_value == fill_value or (
@@ -951,18 +961,23 @@ class ArrayStack(Array):
         else:
             def fill_value_ok(array):
                 return array.fill_value == fill_value
+
         for array in stack.flat:
             if not isinstance(array, Array):
                 raise ValueError('sub-array must be subclass of Array')
-            ok = (array.shape == item_shape and array.dtype == dtype and
-                  fill_value_ok(array))
+            if fill_value is not None and not fill_value_ok(array):
+                fill_value = None
+            ok = array.shape == item_shape and array.dtype == dtype
             if not ok:
                 raise ValueError('invalid sub-array')
 
         self._stack = stack
         self._item_shape = item_shape
         self._dtype = dtype
-        self._fill_value = fill_value
+        if fill_value is None:
+            self._fill_value = np.ma.empty(0, dtype=dtype).fill_value
+        else:
+            self._fill_value = fill_value
 
     @property
     def dtype(self):
@@ -1044,6 +1059,8 @@ class ArrayStack(Array):
             where each Array in the stack must be of the same shape.
 
         """
+        arrays = np.require(arrays, dtype='O')
+
         order = order.lower()
 
         # Ensure a suitable shape has been specified.
@@ -1120,11 +1137,16 @@ class LinearMosaic(Array):
                 raise ValueError('inconsistent tile shapes')
             if tile.dtype != common_dtype:
                 raise ValueError('inconsistent tile dtypes')
-            if not fill_value_ok(tile):
-                raise ValueError('inconsistent tile fill_values')
+            if common_fill_value is not None and not fill_value_ok(tile):
+                common_fill_value = None
+
         self._tiles = tiles
         self._axis = axis
         self._cached_shape = None
+        if common_fill_value is None:
+            self._fill_value = np.ma.empty(0, dtype=common_dtype).fill_value
+        else:
+            self._fill_value = common_fill_value
 
     @property
     def dtype(self):
@@ -1132,7 +1154,7 @@ class LinearMosaic(Array):
 
     @property
     def fill_value(self):
-        return self._tiles[0].fill_value
+        return self._fill_value
 
     @property
     def shape(self):
@@ -1202,7 +1224,13 @@ class LinearMosaic(Array):
                 if isinstance(axis_key, slice) and \
                         axis_key.step is not None and axis_key.step < 0:
                     tiles.reverse()
-                result = LinearMosaic(tiles, axis)
+                # Adjust the axis of the new mosaic to account for any
+                # scalar keys prior to our current mosaic axis.
+                new_axis = axis
+                for key in keys[:axis]:
+                    if _is_scalar(key):
+                        new_axis -= 1
+                result = LinearMosaic(tiles, new_axis)
             else:
                 raise TypeError('invalid key {!r}'.format(axis_key))
 
@@ -1277,16 +1305,25 @@ def _all_slices_inner(item_size, shape, always_slices=False):
             step = MAX_CHUNK_SIZE // nbytes
             slices = []
             for start in range(0, size, step):
-                slices.append(slice(start, start + step))
+                slices.append(slice(start, np.min([start + step, size])))
         nbytes *= size
         all_slices.insert(0, slices)
     return all_slices
 
 
-def save(sources, targets):
+def save(sources, targets, masked=False):
     """
     Save the numeric results of each source into its corresponding target.
 
+    Parameters
+    ----------
+    sources: list
+        The list of source arrays for saving from; limited to length 1.
+    targets: list
+        The list of target arrays for saving to; limited to length 1.
+    masked: boolean
+        Uses a masked array from sources if True.
+
     """
     # TODO: Remove restriction
     assert len(sources) == 1 and len(targets) == 1
@@ -1304,7 +1341,10 @@ def save(sources, targets):
     all_slices = _all_slices(array)
     for index in np.ndindex(*[len(slices) for slices in all_slices]):
         keys = tuple(slices[i] for slices, i in zip(all_slices, index))
-        target[keys] = array[keys].ndarray()
+        if masked:
+            target[keys] = array[keys].masked_array()
+        else:
+            target[keys] = array[keys].ndarray()
 
 
 class _StreamsHandler(object):
@@ -1312,6 +1352,12 @@ class _StreamsHandler(object):
 
     @abstractmethod
     def finalise(self):
+        """
+        Once a chunk has been processed, this method will be called to
+        complete any remaining computation and return a "result chunk"
+        (which itself could very well go on for further processing).
+
+        """
         pass
 
     @abstractmethod
@@ -1330,7 +1376,19 @@ class _AggregationStreamsHandler(_StreamsHandler):
         self.current_keys = None
 
     @abstractmethod
-    def bootstrap(self, data):
+    def bootstrap(self, processed_chunk_shape):
+        """
+        Initialise the processing of the next chunk.
+
+        Parameters
+        ----------
+        processed_chunk_shape : list
+            The shape that the current chunk will have once it has
+            been computed. For example, for an aggregation of a chunk of
+            shape ``(x, y, z)``, over axis 1, ``the processed_chunk_shape``
+            would be ``[x, z]``.
+
+        """
         pass
 
     def input_iteration_order(self, iteration_order):
@@ -1343,12 +1401,17 @@ class _AggregationStreamsHandler(_StreamsHandler):
         keys = list(chunk.keys)
         del keys[self.axis]
         result = None
+        # If this chunk is a new source of data, do appropriate finalisation
+        # of the previous chunk and initialise this one.
         if keys != self.current_keys:
-            shape = list(chunk.data.shape)
-            del shape[self.axis]
-            self.current_shape = shape
+            # If this isn't the first time this method has been called,
+            # finalise any data which is waiting to be dealt with.
             if self.current_keys is not None:
                 result = self.finalise()
+
+            # Setup the processing of this new chunk.
+            shape = list(chunk.data.shape)
+            del shape[self.axis]
             self.bootstrap(shape)
             self.current_keys = keys
         self.process_data(chunk.data)
@@ -1360,8 +1423,8 @@ class _AggregationStreamsHandler(_StreamsHandler):
 
 
 class _CountStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.current_shape = shape
+    def bootstrap(self, processed_chunk_shape):
+        self.current_shape = processed_chunk_shape
         self.running_count = 0
 
     def finalise(self):
@@ -1374,8 +1437,8 @@ class _CountStreamsHandler(_AggregationStreamsHandler):
 
 
 class _CountMaskedStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.running_count = np.zeros(shape, dtype='i')
+    def bootstrap(self, processed_chunk_shape):
+        self.running_count = np.zeros(processed_chunk_shape, dtype='i')
 
     def finalise(self):
         chunk = Chunk(self.current_keys, self.running_count)
@@ -1386,8 +1449,8 @@ class _CountMaskedStreamsHandler(_AggregationStreamsHandler):
 
 
 class _MinStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.result = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.result = np.zeros(processed_chunk_shape, dtype=self.array.dtype)
 
     def finalise(self):
         array = self.result
@@ -1402,8 +1465,8 @@ class _MinStreamsHandler(_AggregationStreamsHandler):
 
 
 class _MinMaskedStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.result = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.result = np.zeros(processed_chunk_shape, dtype=self.array.dtype)
 
     def finalise(self):
         array = self.result
@@ -1418,8 +1481,8 @@ class _MinMaskedStreamsHandler(_AggregationStreamsHandler):
 
 
 class _MaxStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.result = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.result = np.zeros(processed_chunk_shape, dtype=self.array.dtype)
 
     def finalise(self):
         array = self.result
@@ -1434,8 +1497,8 @@ class _MaxStreamsHandler(_AggregationStreamsHandler):
 
 
 class _MaxMaskedStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.result = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.result = np.zeros(processed_chunk_shape, dtype=self.array.dtype)
 
     def finalise(self):
         array = self.result
@@ -1450,8 +1513,9 @@ class _MaxMaskedStreamsHandler(_AggregationStreamsHandler):
 
 
 class _SumStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.running_total = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.running_total = np.zeros(processed_chunk_shape,
+                                      dtype=self.array.dtype)
 
     def finalise(self):
         array = self.running_total
@@ -1466,8 +1530,9 @@ class _SumStreamsHandler(_AggregationStreamsHandler):
 
 
 class _SumMaskedStreamsHandler(_AggregationStreamsHandler):
-    def bootstrap(self, shape):
-        self.running_total = np.ma.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.running_total = np.ma.zeros(processed_chunk_shape,
+                                         dtype=self.array.dtype)
 
     def finalise(self):
         array = self.running_total
@@ -1487,8 +1552,9 @@ class _MeanStreamsHandler(_AggregationStreamsHandler):
         # so it is ignored.
         super(_MeanStreamsHandler, self).__init__(array, axis)
 
-    def bootstrap(self, shape):
-        self.running_total = np.zeros(shape, dtype=self.array.dtype)
+    def bootstrap(self, processed_chunk_shape):
+        self.running_total = np.zeros(processed_chunk_shape,
+                                      dtype=self.array.dtype)
 
     def finalise(self):
         array = self.running_total / self.array.shape[self.axis]
@@ -1507,7 +1573,8 @@ class _MeanMaskedStreamsHandler(_AggregationStreamsHandler):
         self._mdtol = mdtol
         super(_MeanMaskedStreamsHandler, self).__init__(array, axis)
 
-    def bootstrap(self, shape):
+    def bootstrap(self, processed_chunk_shape):
+        shape = processed_chunk_shape
         self.running_count = np.zeros(shape, dtype=self.array.dtype)
         self.running_masked_count = np.zeros(shape, dtype=self.array.dtype)
         self.running_total = np.zeros(shape, dtype=self.array.dtype)
@@ -1548,10 +1615,10 @@ class _StdStreamsHandler(_AggregationStreamsHandler):
         self.ddof = ddof
         super(_StdStreamsHandler, self).__init__(array, axis)
 
-    def bootstrap(self, shape):
+    def bootstrap(self, processed_chunk_shape):
         self.k = 1
         dtype = (np.zeros(1, dtype=self.array.dtype) / 1.).dtype
-        self.q = np.zeros(shape, dtype=dtype)
+        self.q = np.zeros(processed_chunk_shape, dtype=dtype)
 
     def finalise(self):
         self.q /= (self.k - self.ddof)
@@ -1594,12 +1661,16 @@ class _StdMaskedStreamsHandler(_AggregationStreamsHandler):
     def __init__(self, array, axis, ddof):
         self.ddof = ddof
         super(_StdMaskedStreamsHandler, self).__init__(array, axis)
+        self.target_shape = list(self.array.shape)
+        del self.target_shape[self.axis]
 
-    def bootstrap(self, shape):
+    def bootstrap(self, processed_chunk_shape):
         dtype = (np.zeros(1, dtype=self.array.dtype) / 1.).dtype
-        self.a = np.zeros(shape, dtype=dtype).flatten()
-        self.q = np.zeros(shape, dtype=dtype).flatten()
-        self.running_count = np.zeros(shape, dtype=dtype).flatten()
+        self.a = np.zeros(processed_chunk_shape, dtype=dtype).flatten()
+        self.q = np.zeros(processed_chunk_shape, dtype=dtype).flatten()
+        self.running_count = np.zeros(processed_chunk_shape,
+                                      dtype=dtype).flatten()
+        self.current_shape = processed_chunk_shape
 
     def finalise(self):
         mask = self.running_count == 0
